from .predictor import Phasefinder

def version_info():
    return {
        "name": "phasefinder",
        "version": "0.0.2",
        "description": "A beat estimation model that predicts metric position as rotational phase.",
        "author": "bleugreen",
        "license": "AGPL-3.0"
    }
